require 'redcloth'
require 'textile_tags'

class Post
  DIV = /\r?\n---/
  
  class << self
    attr_accessor :base_dir, :glob
    
    def filenames
      Dir.glob File.join(base_dir, Post.glob)
    end
    
    def find_latest
      find.first
    end
    
    alias_method :find_single, :find_latest
    
    def categories
      find.reduce({}) do |tag_score, post| 
        (post[:tags] || []).each do |tag|
          tag_score[tag] ||= 0
          tag_score[tag] += 1
        end
        tag_score
      end.sort
    end
    
    def find_by_tag(tag)
      find.select { |post| (post[:tags] || []).map { |t| t.downcase }.include? tag.downcase }
    end
    
    def find
      filenames \
        .map { |path| from_file(path) } \
        .sort { |p1,p2| p2[:date] <=> p1[:date] }
    end

    def from_file(path)
      date = Date.parse(File.basename(path)[0..9])
      match = path.match(/posts\/(.+)\//)
      template = (match[1] + '_post').to_sym if match
      meta, textile = split_and_parse read_file_strip_bom(path)
      new meta, textile, date, path, template
    end
    
    def from_s(raw, date = nil)
      date ||= Date.today
      meta, textile = split_and_parse raw
      new meta, textile, date
    end

private

    def read_file_strip_bom(path)
      IO.read(path, 3) == "\357\273\277" ? IO.read(path, nil, 3) : IO.read(path)
    end

    def split_and_parse(raw)
      meta, body = {}, ''
      header_body = raw.split(DIV)
      meta = YAML.load(header_body.first) if raw =~ DIV
      body = header_body.last if (header_body.size == 1 and raw !~ DIV) or header_body.size == 2
      [meta, body]
    end
  end
  
  def initialize(meta, textile, date, path = nil, template = nil)
    @meta = meta
    @textile = textile
    @path = path
    @meta['date'] = date
    @meta['template'] = template
  end
  
  def [](property)
    @meta[property.to_s]
  end
  
  def html
    r = RedCloth.new(@textile)
    r.extend VideoTag
    r.to_html
  end
  
  def path
    @path
  end
  
  def slug
    return '#' unless @path
    File.basename(@path, '.textile').sub('-','/').sub('-','/').sub('-','/')
  end
end